\ Zugriff auf On-Chip Flash des MAX10
\ FORTH-Routinen für SmallForth auf K1
\ Autor: Klaus Kohl-Schoepe

\ --------------------------------------
\ I/O Interface zum UFM
\ --------------------------------------
$ff20 Constant #UC      \ Control
$ff22 Constant #UA      \ Addresse
$ff24 Constant #UDL     \ Data Low
$ff26 Constant #UDH     \ Data high
$ff28 Constant #UCC     \ CSR Control
$ff2a Constant #UCA     \ CSR Adresse
$ff2c Constant #UCDL    \ CSR Data low
$ff2e Constant #UCDH    \ CSR Data high
$0100 Constant #U_R     \ Read
$0200 Constant #U_W     \ Write
$0400 Constant #U_V     \ Valid
$0800 Constant #U_H     \ Hold = Wait

\ --------------------------------------

\ --------------------------------------
\ Routinen für Zugriff
\ --------------------------------------
  : 2@uc      ( addr -- datal datah )
    #UCA !   #U_R  #UCC @ $ff and +  #UCC !
    BEGIN #UCC @  #U_V  and UNTIL
    #UCDH @   #UCDL @  swap ;

  : 2!uc      ( datal datah addr -- )
    #UCA !   #UCDH !   #UCDL !
    #U_W  #UCC  @ $ff and +  #UCC !
    BEGIN #UCC @  #U_W and 0= UNTIL ;

  : uidlewait    ( -- f ) \ Warten bis Flash fertig
    BEGIN 0 2@uc drop  3 and  0= UNTIL ;

  : 2@u          ( addr -- datal datah )
    4 / $1fff and  #UA !
    #U_R  #UC @ $ff and +  #UC !
    BEGIN #UC @  #U_V  and UNTIL
    #UDH @   #UDL @  swap ;

  : 2!u          ( datal datah addr -- )
    4 /  $1fff and  #UA !   #UDH !   #UDL !
    uidlewait    
    $FFFF $FE7F 1 2!uc \ Disable Protection
    #U_W  #UC @ $ff and +  #UC !
    BEGIN #UC @  #U_W  and UNTIL
    $FFFF $FFFF 1 2!uc ; \ Enable Protection

  : dumpu       (  addr len -- )
    base @ >r  hex
    0 $10 um/mod swap IF 1+ THEN
    ?FOR cr  $fffc and   dup 4 0u.r space
         4 FOR dup 2@u
               dup flip  2 0u.r  space   2 0u.r  space
               dup flip  2 0u.r  space   2 0u.r  space
               4 +
         NEXT  #16 -
         4 FOR dup 2@u 
               dup flip $ff and dup $20 u< IF drop $20 THEN emit
                        $ff and dup $20 u< IF drop $20 THEN emit
               dup flip $ff and dup $20 u< IF drop $20 THEN emit
                        $ff and dup $20 u< IF drop $20 THEN emit
               4 +
         NEXT
         key? IF key drop  rdrop  0 >r THEN
    NEXT
    drop  r> base ! ;

\ --------------------------------------
\ Löschen und Schreiben der ersten 32K
\ --------------------------------------
  : eraseu      ( -- ) \ Löschen, wenn nicht schon leer
    $FFFF $FE7F 1 2!uc  \ Disable Protection
    $FFFF  $1000 FOR r@ 4 *          2@u and  and NEXT  1+ \ nicht gelöscht ?
    IF
      $FFFF $FF1F 1 2!uc \ Sector 0 erase command
      uidlewait            \ Warten
      0 2@uc drop $10 and 0= ?Abort" Erase error - Sector 1"
    THEN
    $FFFF  $1000 FOR r@ 4 * $1000 +  2@u and  and NEXT  1+ \ nicht gelöscht ?
    IF
      $FFFF $FEAF 1 2!uc \ Sector 1 erase command
      uidlewait            \ Warten
      0 2@uc drop $10 and 0= ?Abort" Erase error - Sector 2"
    THEN
    $FFFF $FFFF 1 2!uc ; \ Enable Protection

   : saveu      ( -- ) \ Programm in UFM speichern
     eraseu
     0   here 3 +  $fffc and  0 4 um/mod nip \ Align auf 4 Byte
     ?FOR dup >r  2@  r@ 2!u  r> 4 + NEXT
     drop ;

   : loadu      ( -- ) \ Programm von UFM laden
     $2000 FOR r@ 4 *  2@u  r@ 4 *  2! NEXT  init ; 

\ --------------------------------------
\ Suchen eines Bitmusters im Flash
\ --------------------------------------
\   : sufm        ( addr n -- addr2 )
\     BEGIN
\       over 2@u >r
\       4 FOR dup >r  $ff and  over  =
\             IF rdrop rdrop rdrop drop exit THEN
\             r>  8 rshift
\         NEXT  drop r>
\       4 FOR dup >r  $ff and  over  =
\             IF rdrop rdrop drop exit THEN
\             r>  8 rshift
\         NEXT drop
\       swap 1+  swap  over 0=
\     UNTIL drop ;
